from ProgressBar import *
from Tkinter import *
import time
import thread

class TkinterProgressBar(ProgressBar):
  """
  The progress bar implementation.
  """
  
  def __init__(self, canvas, displayPercents = False):
    ProgressBar.__init__(self)
    self.canvas = canvas
    self.displayPercents = displayPercents
    self.margin = 10
    self.height = 12
    self.textWidth = 30
    self.color = "red"
    self.outlineColor = "white"
    self.updateGeometry()
    self.createObjects()
    self.canvas.bind("<Configure>", self.onCanvasResize)

  def updateGeometry(self):
    self.canvasDimensions = (self.canvas.winfo_width(), self.canvas.winfo_height())
    top = self.canvasDimensions[1] / 2 - self.height / 2
    self.barBox = [self.margin, top, self.canvasDimensions[0] - 1 - self.margin, top + self.height]
    if self.displayPercents:
       self.barBox[2] -= self.textWidth
    self.barBox.append(self.barBox[2] - self.barBox[0])
    self.barBox.append(self.barBox[3] - self.barBox[1])

  def createObjects(self):
    self.objects = {}
    self.objects["bar_outline"] = self.canvas.create_rectangle(self.barBox[0], self.barBox[1], self.barBox[2], self.barBox[3], fill = "", outline = self.outlineColor)
    innerBox = self.getInnerBarBox()
    self.objects["bar"] = self.canvas.create_rectangle(innerBox[0], innerBox[1], innerBox[2], innerBox[3], fill = self.color, width = 0)
    self.objects["percent_text"] = self.canvas.create_text(self.barBox[2] + 4, self.barBox[1], anchor = NW, text = self.formatPercent(), fill = "white")

  def updateObjects(self):
    """
    Update objects to match the geometry
    """
    self.canvas.coords(self.objects["bar_outline"], self.barBox[0], self.barBox[1], self.barBox[2], self.barBox[3])
    innerBox = self.getInnerBarBox()
    self.canvas.coords(self.objects["bar"], innerBox[0], innerBox[1], innerBox[2], innerBox[3])
    self.canvas.coords(self.objects["percent_text"], self.barBox[2] + 4, self.barBox[1])

  def setValue(self, value):
    ProgressBar.setValue(self, value)
    innerBox = self.getInnerBarBox()
    self.canvas.coords(self.objects["bar"], innerBox[0], innerBox[1], innerBox[2], innerBox[3])
    self.canvas.itemconfigure(self.objects["percent_text"], text = self.formatPercent())

  def getInnerBarBox(self):
    return (self.barBox[0] + 1, 
            self.barBox[1] + 1, 
            self.barBox[0] + 1 + int((self.barBox[4] - 1) * self.getNormalisedValue()),
            self.barBox[3])

  def onCanvasResize(self, event):
    self.updateGeometry()
    self.updateObjects()

  def formatPercent(self):
    if self.displayPercents:
      return ProgressBar.formatPercent(self)
    else:
      return ""
          
class TkinterProgressBarDisplayWindow(Tk):
  """
  Holds the Tk window event handlers and initialisation routines, as well as the progress bar object
  """
  
  def __init__(self, className, displayPercents = False):
    Tk.__init__(self, className = className)
    self.config(bg = "black")
    self.createWidgets()
    self.bar = TkinterProgressBar(self.canvas, displayPercents)
    self.barValue = self.bar.value
    self.updateEvent = "<<UpdateProgressBar>>"
    self.bind(self.updateEvent, self.onUpdateProgressBar)
        
  def createWidgets(self):
    self.canvas = Canvas(self, background = "black", highlightbackground = "black", selectbackground = "black")
    self.canvas.pack(fill = "both", expand = 1)
    self.update_idletasks()

  def setBarValue(self, value):
    """
    Interface to set the the progress bar value from other threads.
    The value is updated in the progress bar when processing an <<UPDATE_PBAR>> virtual event.
    """
    self.barValue = value    

  def onUpdateProgressBar(self, event):
    self.bar.setValue(self.barValue)

class TkinterProgressBarApplication:
  """
  A sample application using TkinterProgressBar.

  Users may override the mainLoop function to do what they want with the progress bar
  """
  
  def __init__(self, className = 'tkpbarapp', displayPercents = False):
    self.displayWindow = TkinterProgressBarDisplayWindow(className, displayPercents)

  def mainLoop(self):
    """
    A dummy example to show how to write the mainLoop function
    """
    self.displayWindow.bar.max = 10
    for i in range(10):
      self.displayWindow.setBarValue(i + 1)
      self.updateDisplayWindow()
      time.sleep(0.25)

  def updateDisplayWindow(self):
    """
    Generate an event to update display window
    """
    self.displayWindow.event_generate(self.displayWindow.updateEvent, when = "tail")

  def run(self):
    thread.start_new_thread(self.mainLoop, ())
    self.displayWindow.mainloop();
    
